﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [StorableClass]
  [Item("GenealogyGraph", "")]
  public class GenealogyGraph : DirectedGraph, IGenealogyGraph {
    [Storable]
    private Dictionary<double, LinkedList<IGenealogyGraphNode>> ranks; // use a linked list for fast insertion/removal
    public Dictionary<double, LinkedList<IGenealogyGraphNode>> Ranks {
      get { return ranks; }
      set { ranks = value; }
    }
    public new IEnumerable<IGenealogyGraphNode> Nodes {
      get { return from n in base.Nodes select (IGenealogyGraphNode)n; }
    }
    protected GenealogyGraph(GenealogyGraph original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GenealogyGraph(this, cloner);
    }
    [StorableConstructor]
    protected GenealogyGraph(bool deserializing) : base(deserializing) { }
    public GenealogyGraph() {
      Ranks = new Dictionary<double, LinkedList<IGenealogyGraphNode>>();
    }
    public override void AddVertex(IVertex vertex) {
      var node = (IGenealogyGraphNode)vertex;
      if (!Ranks.ContainsKey(node.Rank))
        Ranks[node.Rank] = new LinkedList<IGenealogyGraphNode>();
      Ranks[node.Rank].AddLast(node);
      base.AddVertex(vertex);
    }
    public override void RemoveVertex(IVertex vertex) {
      var node = (IGenealogyGraphNode)vertex;
      if (Ranks.ContainsKey(node.Rank)) {
        Ranks[node.Rank].Remove(node);
      }
      base.RemoveVertex(vertex);
    }
    public event EventHandler GraphUpdated;
    private void OnGraphUpdated(object sender, EventArgs args) {
      var updated = GraphUpdated;
      if (updated != null) updated(sender, args);
    }

    public new List<IGenealogyGraphNode> this[object content] {
      get {
        var result = base[content];
        return result != null ? result.Cast<IGenealogyGraphNode>().ToList() : null;
      }
    }

    public override void Clear() {
      base.Clear();
      ranks.Clear();
    }
  }

  [StorableClass]
  [Item("GenealogyGraph", "")]
  public class GenealogyGraph<T> : DirectedGraph, IGenealogyGraph<T> where T : class, IItem {
    // members and properties
    [Storable]
    private Dictionary<double, LinkedList<IGenealogyGraphNode>> ranks;
    public Dictionary<double, LinkedList<IGenealogyGraphNode>> Ranks {
      get { return ranks; }
      set { ranks = value; }
    }
    public new IEnumerable<IGenealogyGraphNode<T>> Nodes {
      get { return from n in base.Nodes select (IGenealogyGraphNode<T>)n; }
    }
    // contructors
    protected GenealogyGraph(GenealogyGraph<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GenealogyGraph<T>(this, cloner);
    }
    [StorableConstructor]
    protected GenealogyGraph(bool deserializing) : base(deserializing) { }
    public GenealogyGraph() {
      Ranks = new Dictionary<double, LinkedList<IGenealogyGraphNode>>();
    }
    // methods
    public override void AddVertex(IVertex vertex) {
      var node = (IGenealogyGraphNode<T>)vertex;
      if (!Ranks.ContainsKey(node.Rank))
        Ranks[node.Rank] = new LinkedList<IGenealogyGraphNode>();
      Ranks[node.Rank].AddLast(node);
      base.AddVertex(vertex);
    }
    public override void RemoveVertex(IVertex vertex) {
      var node = (IGenealogyGraphNode<T>)vertex;
      if (Ranks.ContainsKey(node.Rank)) {
        Ranks[node.Rank].Remove(node);
      }
      base.RemoveVertex(vertex);
    }

    IEnumerable<IGenealogyGraphNode> IGenealogyGraph.Nodes {
      get { return Nodes; }
    }

    public event EventHandler GraphUpdated;
    private void OnGraphUpdated(object sender, EventArgs args) {
      var updated = GraphUpdated;
      if (updated != null) updated(sender, args);
    }
    public new List<IGenealogyGraphNode<T>> this[object content] {
      get {
        var result = base[content];
        return result != null ? result.Cast<IGenealogyGraphNode<T>>().ToList() : null;
      }
    }
  }
}
